button = {}
local bm = {__index = button}

local fade_speed = 6

--Local functions
local function setAlpha(a)
	local r,g,b,oa = love.graphics.getColor()
	love.graphics.setColor(r, g, b, a)
end

--Global
function button.new(func, text, x, y, width, height, color, font, font_color, overlay_color, overlay_alpha)
	overlay_color = overlay_color or {32, 32, 32}
	overlay_alpha = overlay_alpha or 255
	color[4] = color[4] or 255
	font_color[4] = font_color[4] or 255
	local b = {
		func = func,
		arg = arg,
		text = text,
		x = x,
		y = y,
		width = width, 
		height = height,
		color = color,
		font = font,
		font_color = font_color,
		mouse_over = false,
		overlay_color = overlay_color,
		overlay_alpha = overlay_alpha,
		_overlay_alpha = 0,
		pressed = false
	}
	return setmetatable(b, bm)
end

function button:update(dt)
	local mx, my = love.mouse.getPosition()
	self.mouse_over = false
	if mx > self.x and mx < (self.x + self.width) and my > self.y and my < (self.y + self.height) then
		self.mouse_over = true
	end

	--Hover overlay
	if self.mouse_over then
		self._overlay_alpha = self._overlay_alpha + fade_speed * dt
		if self._overlay_alpha > 1 then
			self._overlay_alpha = 1
		end
	else
		self._overlay_alpha = self._overlay_alpha - fade_speed * dt
		if self._overlay_alpha < 0 then
			self._overlay_alpha = 0
		end
	end

	--Pressing
	if self.mouse_over then
		if love.mouse.isDown("l") then
			if not self.pressed then
				self.func(self.text)
				self.pressed = true
			end
		else
			self.pressed = false
		end
	end
end

function button:draw()
	--Body
	love.graphics.setColor(self.color)
	love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)

	--overlay
	love.graphics.setColor(self.overlay_color)
	setAlpha(self.overlay_alpha * self._overlay_alpha)
	love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)

	--Text
	local text_x, text_y
	text_x = self.x + (self.width / 2) - (self.font:getWidth(self.text) / 2)
	text_y = self.y + (self.height / 2) - (self.font:getHeight() / 2)
	love.graphics.setColor(self.font_color)
	love.graphics.setFont(self.font)
	love.graphics.print(self.text, text_x, text_y)


	love.graphics.setColor(color.button.outline)
	love.graphics.rectangle("line", self.x, self.y, self.width, self.height)
end