local screen = {
		width = love.graphics.getWidth(),
		height = love.graphics.getHeight()
	}

font_size = 12
font = {
	normal = love.graphics.newFont(font_size),
}


console = {
	textbox = {
		text = "",
		history = {},
		text_x = 0,
		text_y = 0,
		text_color = {200, 200, 200, 255},
		box_color = {16, 16, 16, 255},
		box_border = 8,
		delete_speed = 0.05,
		delete_tick = 0.05,
		delete_repeate_delay = 0.5,
		delete_repeate_delay_tick = 0.5,
		indicator = {
			x = 0,
			alpha = 1,
			t = 0,
			flash_speed = 4,
			character = "|",
			color = {200, 200, 200, 255},
			distance = 1
		},
		history = {},
		history_i = -1
	},
	history = {
		background_color = {32, 32, 32, 128},
		text_color = {150, 150, 150, 128},
		text = {},
		x = 0,
		y = 0,
		box_border = 8,
		vertical_spacing = 4
	},
	visible = false,
	enabled = false
}

function console:initialize()
	self.textbox.text_x = self.textbox.box_border
	self.textbox.text_y = screen.height - font_size - (self.textbox.box_border / 2)

	self.history.x = self.history.box_border
	self.history.y = self.textbox.text_y - self.textbox.box_border - font.normal:getHeight()
end

function console:textinput(t)
	if self.visible then
		self.textbox.text = self.textbox.text..t
	end
end

function console:update(dt)
	if self.visible then
		if love.keyboard.isDown("backspace") then
			self.textbox.delete_repeate_delay_tick = self.textbox.delete_repeate_delay_tick - dt
			if self.textbox.delete_repeate_delay_tick < 0 then
				self.textbox.delete_tick = self.textbox.delete_tick + dt
				if self.textbox.delete_tick > self.textbox.delete_speed then
					console:backspace()
					self.textbox.indicator.alpha = 1
					self.textbox.indicator.t = 1
					self.textbox.delete_tick = 0
				end
			end
		else
			self.textbox.delete_tick = self.textbox.delete_speed
			self.textbox.delete_repeate_delay_tick = self.textbox.delete_repeate_delay
		end

		--text indicator
		local i = self.textbox.indicator
		i.t = i.t + i.flash_speed * dt
		i.alpha = math.abs(math.sin(i.t))
		i.x = self.textbox.text_x + font.normal:getWidth(self.textbox.text) + i.distance
	end
end

function console:draw()
	if self.visible then
		local old_font = love.graphics.getFont()
		local old_r, old_g, old_b, old_a = love.graphics.getColor()

		--Background
		love.graphics.setColor(self.history.background_color)
		love.graphics.rectangle("fill", 0, 0, screen.width, screen.height)

		--box
		love.graphics.setColor(self.textbox.box_color)
		love.graphics.rectangle("fill", 0, screen.height - font_size - self.textbox.box_border, screen.width, font_size + self.textbox.box_border)
		--text
		love.graphics.setColor(self.textbox.text_color)
		love.graphics.setFont(font.normal)
		love.graphics.print(self.textbox.text, self.textbox.text_x, self.textbox.text_y)
		--indicator
		love.graphics.setColor(self.textbox.indicator.color[1], self.textbox.indicator.color[2], self.textbox.indicator.color[3], self.textbox.indicator.color[4] * self.textbox.indicator.alpha)
		love.graphics.print(self.textbox.indicator.character, self.textbox.indicator.x, self.textbox.text_y)

		--History
		love.graphics.setColor(self.history.text_color)
		local t = #self.history.text
		for i,v in ipairs(self.history.text) do
			t = t - 1
			local y = self.history.y - ((font.normal:getHeight() + self.history.vertical_spacing) * t)
			love.graphics.print(v.text, self.history.x, y)
		end
		--Resetting things
		love.graphics.setFont(old_font)
		love.graphics.setColor(old_r, old_g, old_b, old_a)
	end
end

function console:keypressed(key)
	if self.visible then
		if key == "backspace" then
			console:backspace()
		elseif key == "return" then
			self.textbox.history[#self.textbox.history + 1] = self.textbox.text
			console:execute(self.textbox.text)
			self.textbox.history_i = -1
		elseif key == "up" then
			self.textbox.history_i = self.textbox.history_i + 1
			if self.textbox.history_i > (#self.textbox.history - 1) then
				self.textbox.history_i = #self.textbox.history - 1
			end
			self.textbox.text = self.textbox.history[#self.textbox.history - self.textbox.history_i]
		end
		self.textbox.indicator.alpha = 1
		self.textbox.indicator.t = 1
	end
end

function console:backspace()
	if #self.textbox.text > 0 then
		self.textbox.text = self.textbox.text:sub(1, -2)
		if #self.textbox.text > 0 then
			if self.textbox.text:sub(-1, -1):byte() >= 129 then 
				self.textbox.text = self.textbox.text:sub(1, -2)
			end
		end
	end
end

function console:print(t)
	self.history.text[#self.history.text + 1] = {
		text = t
	}
	self.textbox.text = ""
end

function console:execute(t)
	local f, e = loadstring(t)
	local status, err = pcall(f)
	if status then
		f()
	else
		self:print(err)
	end

	self.textbox.text = ""
end

function console:toggle(status)
	if enabled then
		status = status or not self.visible
		self.visible = status
	end
end

function console:isVisible()
	return self.visible
end

function console:resize(w, h)
	screen.width = w
	screen.height = h
	self:initialize()
end