screen = {
		width = love.graphics.getWidth(),
		height = love.graphics.getHeight()
	}

--loading files
require "color"
require "button"
require "console"

local button_font = love.graphics.newFont(24)
local display_font = love.graphics.newFont("font_screen.ttf", 24)

--Calclator variables
formula = ""

--Buttons
button_layout = {
	{"7", "8", "9", "<"},
	{"4", "5", "6", "*"},
	{"1", "2", "3", "-"},
	{".", "0", "/", "+"},
	{"=", "", "C", ""}

}

local button_spacing = 4
local buttons = {}
local display_offset = 8
local display = {}

function button_pressed(arg)
	if arg == "0" then
		formula = formula..0
	elseif arg == "1" then
		formula = formula..1
	elseif arg == "2" then
		formula = formula..2
	elseif arg == "3" then
		formula = formula..3
	elseif arg == "4" then
		formula = formula..4
	elseif arg == "5" then
		formula = formula..5
	elseif arg == "6" then
		formula = formula..6
	elseif arg == "7" then
		formula = formula..7
	elseif arg == "8" then
		formula = formula..8
	elseif arg == "9" then
		formula = formula..9
	elseif arg == "." then
		formula = formula.."."
	elseif arg == "/" then
		formula = formula.."/"
	elseif arg == "*" then
		formula = formula.."*"
	elseif arg == "-" then
		formula = formula.."-"
	elseif arg == "+" then
		formula = formula.."+"
	elseif arg == "C" then
		formula = ""
	elseif arg == "<" then
		formula = formula:sub(1, -2)
	elseif arg == "=" then
		if #formula > 0 then
			local c, e = loadstring("return tostring("..formula..")")
			if c then formula = c() end
		end
	end
end

function create_gui()
	buttons = {}
	display = {
		x = display_offset,
		y = display_offset,
		width = screen.width - (display_offset * 2),
		height = screen.height / 4,
	}
	display_font = love.graphics.newFont("font_screen.ttf", display.height / 3)

	local button_width = (screen.width / 4) - (button_spacing * 1.8)
	local button_height = (screen.height - (display.height + display_offset)) / #button_layout - (button_spacing * 2)

	for _y=1, #button_layout do
		for _x=1, #button_layout[1] do
			local x = ((button_width + button_spacing) * _x) - button_width + button_spacing
			local y = ((display.height + display_offset) + ((button_height + button_spacing) * _y)) - button_height + display_offset
			local w = button_width
			if button_layout[_y][_x] == "=" or button_layout[_y][_x] == "C" then
				w = (screen.width / 2) - (button_spacing * 2.5)
			end
			if button_layout[_y][_x] ~= "" then
				buttons[#buttons + 1] = button.new(button_pressed, button_layout[_y][_x], x, y, w, button_height, color.button.background, button_font, color.button.text)
			end
		end
	end

	button_font = love.graphics.newFont((buttons[1].width + buttons[1].height) / 5)
end


function love.load()
	love.graphics.setBackgroundColor(color.window.background)
	create_gui()
	console:initialize()
end

function love.update(dt)
	if not console:isVisible() then
		for i,v in ipairs(buttons) do
			v:update(dt)
		end
	end
	console:update(dt)
end

function love.draw()
	--Display
	love.graphics.setColor(color.display.background)
	love.graphics.rectangle("fill", display.x, display.y, display.width, display.height)
	love.graphics.setColor(color.display.outline)
	love.graphics.rectangle("line", display.x, display.y, display.width, display.height)

	love.graphics.setColor(color.display.text)
	love.graphics.setFont(display_font)
	love.graphics.print(formula, display.x + display_offset, display.y + display_offset)

	--Buttons
	for i,v in ipairs(buttons) do
		v:draw()
	end
	console:draw()
end

function love.textinput(t)
	console:textinput(t)
end

function love.keypressed(key)
	console:keypressed(key)
	if key == "escape" then love.event.push("quit") end

	if not console:isVisible() then
		if key == "kp0" or key == "0" then
			button_pressed("0")
		elseif key == "kp1" or key == "1"  then
			button_pressed("1")
		elseif key == "kp2" or key == "2"  then
			button_pressed("2")
		elseif key == "kp3" or key == "3"  then
			button_pressed("3")
		elseif key == "kp4" or key == "4"  then
			button_pressed("4")
		elseif key == "kp5" or key == "5"  then
			button_pressed("5")
		elseif key == "kp6" or key == "6"  then
			button_pressed("6")
		elseif key == "kp7" or key == "7"  then
			button_pressed("7")
		elseif key == "kp8" or key == "8"  then
			button_pressed("8")
		elseif key == "kp9" or key == "9"  then
			button_pressed("9")
		elseif key == "kp/" then
			button_pressed("/")
		elseif key == "kp*" then
			button_pressed("*")
		elseif key == "kp-" then
			button_pressed("-")
		elseif key == "kp+" then
			button_pressed("+")
		elseif key == "kp," or key == "kp." then
			button_pressed(".")
		elseif key == "kp." then
			button_pressed(".")
		elseif key == "kpenter" or key == "kp=" then
			button_pressed("=")
		elseif key == "backspace" then
			button_pressed("<")
		elseif key == "delete" then
			button_pressed("C")
		end
	end

	if key == "tab" then console:toggle() end
end

function love.resize(w, h)
	console:resize(w, h)
	screen.width = w
	screen.height = h
	create_gui()
end